import bucketTemplate from './bucket.template.html'

/* @ngInject */
let bucketComponent = {
  template: bucketTemplate,
  bindings: {
    questionsId: '=',
    sid: '='
  },
  transclude: true,
  controller: function ($rootScope, $scope, $http, $timeout, $state, $localStorage, $stateParams, $document, $log, xxtea, exerciseService, appConstants, popupService, loaderService) {
    // Local config
    $scope.config = {}

    /**
         * Error callback for HTTP response
         * @param {object} response The HTTP response object
         * @returns {void}
         */
    function errorCallback(response) {
      if (response.status === 401) {
        $log.error({ status: 'false', data: 'Bad token.' })
      } else if (response.status === 404) {
        $log.error({ status: 'false', data: 'Bad request.' })
      } else if (response.status === 405) {
        $log.error({ status: 'false', data: 'Method not allowed.' })
      } else {
        $log.error({ status: 'false', data: response.statusText })
      }

    };

    // Current question ( step )
    $scope.step = 0
    $scope.subStep = 0

    // User info
    $scope.info = {
      time_attempted: new Date().getTime(),
      time_finished: undefined
    }
    // Questions related data
    $scope.current = {
      question: undefined,
      correct: undefined,
      selected: undefined,
      isCorrect: undefined,
      answers: [],
      attempts: 0,
      tryAgain: false,
      nextQuestion: false
    }

    // Popup
    $scope.popupVisible = false
    let popupVisibilityChange = $rootScope.$on('popup.changed', function (elem, data) {
      $scope.popupVisible = data.active
    })
    /**
         * Reset state of the exercise to beginning
         * @param {boolean} tryAgain Determines if the action should be try again or next question
         * @returns {void}
         */
    $scope.resetQuestion = (tryAgain) => {
      popupService.hide()
      $scope.current.tryAgain = false
      $scope.current.nextQuestion = false
      $scope.current.isCorrect = undefined
      $scope.current.selected = undefined
      if (!tryAgain) {
        $scope.current.attempts = 0
      }
    }

    /**
         * Setting up the exercise
         * @param {object} exerciseData exercise object
         * @returns {void}
         */
    function setupExercise(exerciseData) {
      $scope.exercise = exerciseData
      $scope.question = $scope.exercise.interaction.questions[$scope.step]
      if (angular.isUndefined($scope.question.answered)) {
        $scope.question.answered = ''
      }
      $scope.current.correct = $scope.exercise.interaction.correct // Decrypt xxTea and parse JSON
      $scope.current.answers = $scope.exercise.interaction.questions.map((item) => { return { question: item.question_id, attempts: [] } }) // Map answers to correct array
      $localStorage.questionCount = $scope.exercise.interaction.questions.length // Get questions count
      stepWatch()
    }

    // Watching function
    function stepWatch() {
      popupService.hide()
      if ($scope.exercise) {
        if ($scope.exercise.interaction.questions.length <= $scope.step && $scope.current.question.questions.length <= $scope.subStep) {
          $scope.finishExercise()
        } else {
          $scope.resetQuestion(false)
          $scope.current.question = $scope.exercise.interaction.questions[$scope.step]
          $localStorage.currentActiveQuestion = $scope.step + 1
        }
      }
    };

    function subStepWatch() {
      popupService.hide()
      if ($scope.exercise) {
        if ($scope.current.question.questions.length <= $scope.subStep) {
          $scope.step++
        }
        $scope.resetQuestion()
      }
    };

    $scope.$watch('step', (n, o) => { stepWatch() }) // Watching for current answer ( step )
    $scope.$watch('subStep', (n, o) => { subStepWatch() })

    $scope.$watch(function () {
      let panel = angular.element($document[0].getElementById('panel-component'))[0]
      return panel ? panel.offsetHeight : 0
    }, function (newHeight, oldHeight) {
      $scope.popupHeight = newHeight
    }, true)

    /**
         * Switch to next question
         * @returns {void}
         */
    $scope.nextQuestion = () => { $scope.step++ }

    /**
         * Switch to next sub question
         * @returns {void}
         */
    $scope.nextSubQuestion = () => { $scope.subStep++; $scope.resetQuestion() }

    /**
         * Check if the current question is last
         * @returns {boolean} true if the question is last
         */
    $scope.isLastQuestion = () => {
      return $scope.current.question ? $scope.current.question.questions.length - 1 === $scope.subStep : false
    }

    /**
         * Checks current action that have to be displayed
         * @param {string} type text or function
         * @returns {object} string or function
         */
    $scope.currentAction = (type) => {
      let attempt = $scope.current.answers.find((a) => { return a.question === $scope.current.question.question_id })
      let attemptsCount = attempt ? attempt.attempts.filter((a) => { return a.answer_index === $scope.subStep }).length : 0
      let attemptHasCorrect = attempt ? attempt.attempts.find((a) => { return a.answer_index === $scope.subStep && a.correct }) : false
      let possibleAttemptsCount = $scope.current.question && $scope.current.question.answers ? $scope.current.question.answers.length - 1 : 1
      if ($scope.current.tryAgain && !$scope.current.nextQuestion) {
        return type === 'text' ? 'Try again' : $scope.resetQuestion(true)
      } else if ($scope.current.nextQuestion) {
        return type === 'text' ? 'Next' : $scope.nextSubQuestion()
      } else {
        return type === 'text' ? 'Check' : $scope.checkQuestion()
      }
    }

    $scope.nextAction = (type) => {
      if ($scope.isLastQuestion()) {
        return type === 'text' ? 'Submit' : $scope.nextSubQuestion()
      } else {
        return type === 'text' ? 'Next' : $scope.nextSubQuestion()
      }
    }

    /**
         * Checks answer
         * @returns {void}
         */
    $scope.checkQuestion = () => {
      let correctArray = $scope.current.correct.find((m, i) => { return m.question_id === $scope.current.question.question_id })
      let isCorrect = xxtea.encryptToBase64($scope.current.question.answers[$scope.current.selected] || '', appConstants.tea) === correctArray.answers[$scope.subStep]

      let userAnswer = xxtea.encryptToBase64($scope.current.question.answers[$scope.current.selected] || '', appConstants.tea)
      let attempt = $scope.current.answers.find((a) => { return a.question === $scope.current.question.question_id }) // Finds array with all user attempts for current question
      attempt.attempts.push({ answer: userAnswer, correct: isCorrect, attempt_number: attempt.attempts.length + 1, answer_index: $scope.subStep })// Adds the attempt to an attempts array
      let attemptsCount = attempt.attempts.filter((a) => { return a.answer_index === $scope.subStep }).length
      $scope.current.isCorrect = isCorrect ? 'yes' : 'no'
      if (angular.isDefined($scope.question.answered) && $scope.question.answered.indexOf($scope.question.questions[$scope.subStep]) !== -1) {
        popupService.show('You have already answered this question!', 'semi-smile', false)
      } else {
        if (isCorrect) {
          popupService.show('Very good!', 'happy-smile', true) // Show popup
          $scope.current.nextQuestion = true
          $scope.question.answered += $scope.question.questions[$scope.subStep]
        } else { // Answer is not correct
          let possibleAttemptsCount = $scope.current.question.answers.length - 1

          if (attemptsCount === possibleAttemptsCount) {
            $scope.question.answered += $scope.question.questions[$scope.subStep]
            popupService.show('The correct answer is: <br/>' + xxtea.decryptFromBase64(correctArray.answers[$scope.subStep], appConstants.tea), 'sad-smile', false) // Show popup
            $scope.current.tryAgain = false
            $scope.current.nextQuestion = true
          } else if (attemptsCount === possibleAttemptsCount - 1) {
            popupService.show('Final chance!', 'semi-smile', false) // Show popup
            $scope.current.tryAgain = true
            $scope.current.nextQuestion = false
          } else {
            popupService.show('Some of your answers are incorrect.', 'semi-smile', false) // Show popup
            $scope.current.tryAgain = true
            $scope.current.nextQuestion = false
          }
        }
      }
    }

    /**
         * Sends an POST request to score API and finishes the exercise
         * @returns {void}
         */
    $scope.finishExercise = () => {
      $scope.info.time_finished = new Date().getTime() // Get time when user finished exercise
      var correctAnswersCount = 0 // Correct answers counter
      // Get correct answers count
      for (var i = 0; i < $scope.current.answers.length; i++) {
        var correctCount = $scope.current.answers[i].attempts.filter(function (item) { return item.correct }).length // Get only correct answers
        correctAnswersCount += correctCount
      }

      let questionsCount = 0
      angular.forEach($scope.exercise.interaction.questions, (v) => {
        questionsCount += v.questions.length
      })

      const score = Math.round(((correctAnswersCount / questionsCount) * 100)) // Get user score in percent
      const data = {
        'score': score,
        'user': 24275,
        'answers': $scope.current.answers,
        'time_attempted': $scope.info.time_attempted,
        'time_finished': $scope.info.time_finished,
        'exercise_id': $scope.exercise.exercise_id,
        'playlist_id': $scope.exercise.playlist.playlist_id,
        'context': $scope.exercise.context.context,
        'context_id': $scope.exercise.context.context_id,
        'id': $scope.exercise.exercise_id,
        'group_id': $scope.exercise.group_id
      }

      // $http.post(appConstants.api + '/exerciseStoreScore', data).then(() => {
      //   let nextExercise = undefined
      //   if ($localStorage.playlist) {
      //     if ($localStorage.currentActiveExercise !== $localStorage.exerciseCount) {
      //       nextExercise = $localStorage.playlist[$localStorage.currentActiveExercise].exercise_id
      //     }
      //   }
      //
      //   exerciseService.setVal({
      //     'score': score,
      //     'totalQuestions': questionsCount,
      //     'correctAnswer': correctAnswersCount,
      //     'nextExercise': nextExercise
      //   })
      //
      //   $state.go('result') // Go to result page
      // }, $scope.errorCallback)

      let nextExercise = undefined
      exerciseService.setVal({
        'score': score,
        'totalQuestions': questionsCount,
        'correctAnswer': correctAnswersCount,
        'nextExercise': nextExercise
      })

      $state.go('result') // Go to result page

    }

    $scope.$watch('[$ctrl.questionsId,$ctrl.sid]', (n, o) => {
      if ($stateParams.id) {
        // Get exercise data and setup
        $http({ method: 'GET', url: appConstants.api + '/exercise/' + $scope.$ctrl.questionsId + '/' + $scope.$ctrl.sid  })
          .then((response) => {
            setupExercise(response.data)
            $localStorage.$reset()
            $localStorage.exercises = []
            $localStorage.questionCount = $scope.exercise.interaction.questions.length
            $localStorage.questions = $scope.exercise.interaction.questions
            $localStorage.currentActiveExercise = 1
            $localStorage.currentActiveQuestion = $scope.step + 1

            // Assign new data
            $localStorage.playlist = [response.data]
            $localStorage.exerciseCount = $localStorage.playlist.length
            $localStorage.currentExercise = response.data.exercise_id
            $localStorage.currentActiveExercise = 1
            $localStorage.questionLink = { type: 'bucket', id: $scope.$ctrl.qid, sid: $scope.$ctrl.sid }

            // $timeout(() => { $state.go('exercise') })
          }, errorCallback)
      } else { // If we have loaded playlist then load the exercise from playlist
        setupExercise($localStorage.playlist.find((item) => item.exercise_id === $localStorage.currentExercise))
      }
    })
  }

}
export default bucketComponent
