import mixTemplate from './mix.template.html'

/* @ngInject */
let mixComponent = {
  template: mixTemplate,
  bindings: {
    questionsId: '=',
    sid: '='
  },
  controller: function ($rootScope, $scope, $http, $timeout, $state, $localStorage, $stateParams, $log, $document, xxtea, exerciseService, appConstants, popupService, loaderService) {

    // Local config
    $scope.config = {
      // SortableJS config
      sortable: {
        disabled: false,
        animation: 0,
        group: 'tiles',
        forceFallback: true,
        ghostClass: 'sortable-chosen',
        delay: 0,
        fallbackTolerance: 0,
        onUpdate: function () {
          $scope.tilesChanged = true
        },
        onAdd: function (evt) {
          $scope.tilesChanged = true
        }
      }
    }

    /**
         * Shuffles the given array
         * @param {array<string>} a The input array
         * @returns {void}
         */
    function shuffleQuestions(a) {
      var j, x, i
      for (i = a.length; i; i--) {
        j = Math.floor(Math.random() * i)
        x = a[i - 1]
        a[i - 1] = a[j]
        a[j] = x
      }
    }

    /**
         * Error callback for HTTP response
         * @param {object} response The HTTP response object
         * @returns {void}
         */
    function errorCallback(response) {
      if (response.status === 401) {
        $log.error({
          status: 'false',
          data: 'Bad token.'
        })
      } else if (response.status === 404) {
        $log.error({
          status: 'false',
          data: 'Bad request.'
        })
      } else if (response.status === 405) {
        $log.error({
          status: 'false',
          data: 'Method not allowed.'
        })
      } else {
        $log.error({
          status: 'false',
          data: response.statusText
        })
      }

    };

    // Popup
    $scope.popupVisible = false
    let popupVisibilityChange = $rootScope.$on('popup.changed', function (elem, data) {
      $scope.popupVisible = data.active
    })
    // Current question ( step )
    $scope.step = 0
    // User info
    $scope.info = {
      time_attempted: new Date().getTime(),
      time_finished: undefined
    }
    // Questions related data
    $scope.current = {
      question: undefined,
      correct: undefined,
      answers: [],
      correctAnswerString: undefined,
      nextQuestion: false,
      tryAgain: false,
      isCorrect: false
    }

    $scope.tilesChanged = false

    /**
         * Checks current action that have to be displayed
         * @param {string} type text or function
         * @returns {object} string or function
         */
    $scope.currentAction = (type) => {
      let attempt = $scope.current.answers.find((a) => { return a.question === $scope.current.question.question_id })
      let attemptHasCorrect = attempt ? attempt.attempts.find((a) => { return a.correct }) : false
      if ($scope.current.tryAgain && !$scope.current.nextQuestion) {
        return type === 'text' ? 'Try again' : $scope.resetQuestion()
      } else if ($scope.current.nextQuestion) {
        return type === 'text' ? 'Next' : $scope.nextQuestion()
      } else {
        return type === 'text' ? 'Check' : $scope.checkQuestion()
      }
    }

    $scope.nextAction = (type) => {
      if ($scope.isLastQuestion()) {
        return type === 'text' ? 'Submit' : $scope.nextQuestion()
      }
    }

    /**
         * Reset state of the exercise
         * @returns {void}
         */
    $scope.resetQuestion = () => {
      popupService.hide()
      $scope.current.tryAgain = false
      $scope.current.nextQuestion = false
      $scope.current.isCorrect = false
      $scope.config.sortable.disabled = false
      //$scope.tilesChanged = false;
    }

    //  Setup exercise
    function setupExercise(exerciseData) {
      $scope.exercise = exerciseData
      $scope.current.correct = $scope.exercise.interaction.correct // Decrypt xxTea and parse JSON
      $scope.current.answers = $scope.exercise.interaction.questions.map((item) => { return { question: item.question_id, attempts: [] } }) // Map answers to correct array
      $localStorage.questionCount = $scope.exercise.interaction.questions.length // Get questions count
      stepWatch()
    }

    // Watching function
    function stepWatch() {
      popupService.hide()
      if ($scope.exercise) {
        if ($scope.exercise.interaction.questions.length <= $scope.step) {
          $scope.finishExercise()
        } else {
          $scope.current.question = $scope.exercise.interaction.questions[$scope.step]
          $scope.current.question.extra_tiles = []
          $scope.current.correctAnswerString = false
          $scope.current.nextQuestion = false
          $scope.current.isCorrect = false
          $scope.config.sortable.disabled = false
          $scope.tilesChanged = false

          shuffleQuestions($scope.current.question.tiles)
          $localStorage.currentActiveQuestion = $scope.step + 1
        }
      }
    }
    $scope.$watch('step', (n, o) => { stepWatch() }) // Watching for current answer ( step )

    /**
         * Switch to next question
         * @returns {void}
         */
    $scope.nextQuestion = () => { $scope.step++ }

    /**
         * Check if the current question is last
         * @returns {boolean} true if the question is last
         */
    $scope.isLastQuestion = () => {
      return $scope.current.question ? $scope.exercise.interaction.questions.length - 1 === $scope.step : false
    }

    /**
         * Check current question answer
         * @returns {void}
         */
    $scope.checkQuestion = () => {
      let userAnswer = xxtea.encryptToBase64($scope.current.question.tiles.join(' '), appConstants.tea) // Get user answer
      let correctAnswers = $scope.current.correct.find((correctQuestion) => correctQuestion.question_id === $scope.current.question.question_id) // Get correct answers
      let isCorrectAnswer = correctAnswers.answers.includes(userAnswer) // Get if answer is correct
      /* Make an attempt */
      let attempt = $scope.current.answers.find((attemptAnswer) => { return attemptAnswer.question === $scope.current.question.question_id }) // Finds array with all user attempts for current question
      attempt.attempts.push({ answer: [userAnswer], correct: isCorrectAnswer, attempt_number: attempt.attempts.length + 1, answer_index: $scope.step })// Adds the attempt to an attempts array
      let attemptsCount = attempt.attempts.length // Attempts count
      let showCorrectAnswer = false // Show correct answer

      // Check if answer is correct
      if ($scope.current.question.answered === true) {
        popupService.show('You have already answered this question!', 'semi-smile', false)
      } else {
        if (isCorrectAnswer) { // Answer is correct
          popupService.show('Very good!', 'happy-smile', true) // Show popup
          showCorrectAnswer = true // Set variable to show correct answer
          $scope.current.isCorrect = true
          $scope.current.question.answered = true
        } else { // Answer is not correct
          if (attemptsCount === 1) {
            popupService.show('Some of your answers are incorrect.<div class="visible-xs visible-sm"></div> They have been left in place.', 'semi-smile', false) // Show popup
            $scope.current.tryAgain = true
            $scope.current.nextQuestion = false
            $scope.current.isCorrect = false
          } else if (attemptsCount === 2) {
            popupService.show('Some of your answers are incorrect.<div class="visible-xs visible-sm"></div> They have been left in place.', 'semi-smile', false) // Show popup
            $scope.current.tryAgain = true
            $scope.current.nextQuestion = false
            $scope.current.isCorrect = false
          } else if (attemptsCount === 3) {
            popupService.show('The correct answers are: <br/>' + xxtea.decryptFromBase64(correctAnswers.answers[0], appConstants.tea), 'sad-smile', false) // Show popup
            showCorrectAnswer = true // Set variable to show correct answer
            $scope.current.tryAgain = false
            $scope.current.nextQuestion = true
            $scope.current.isCorrect = false
            $scope.current.question.answered = true
          }
        }
      }

      $scope.config.sortable.disabled = true

      // Determine to show correct answer
      if (showCorrectAnswer) {
        $scope.current.correctAnswerString = xxtea.decryptFromBase64(isCorrectAnswer ? userAnswer : correctAnswers.answers[0], appConstants.tea) // Set the correct answer
        $scope.current.nextQuestion = true // If true the Next answer button will appear
      }
    }

    /**
         * Sends an POST request to score API and finishes the exercise
         * @returns {void}
         */
    $scope.finishExercise = () => {
      $scope.info.time_finished = new Date().getTime() // Get time when user finished exercise
      var correctAnswersCount = 0 // Correct answers counter
      // Get correct answers count
      for (var i = 0; i < $scope.current.answers.length; i++) {
        var correctCount = $scope.current.answers[i].attempts.filter(function (item) { return item.correct }).length // Get only correct answers
        if (correctCount > 0) { correctAnswersCount++ }
      }

      const score = Math.round(((correctAnswersCount / $scope.exercise.interaction.questions.length) * 100)) // Get user score in percent
      const data = {
        'score': score,
        'user': 24275,
        'answers': $scope.current.answers,
        'time_attempted': $scope.info.time_attempted,
        'time_finished': $scope.info.time_finished,
        'exercise_id': $scope.exercise.exercise_id,
        'playlist_id': $scope.exercise.playlist.playlist_id,
        'context': $scope.exercise.context.context,
        'context_id': $scope.exercise.context.context_id,
        'id': $scope.exercise.exercise_id,
        'group_id': $scope.exercise.group_id
      }

      // $http.post(appConstants.api + '/exerciseStoreScore', data).then(() => {
      //   let nextExercise = undefined
      //   if ($localStorage.playlist) {
      //     if ($localStorage.currentActiveExercise !== $localStorage.exerciseCount) {
      //       nextExercise = $localStorage.playlist[$localStorage.currentActiveExercise].exercise_id
      //     }
      //   }
      //
      //   exerciseService.setVal({
      //     'score': score,
      //     'totalQuestions': $scope.exercise.interaction.questions.length,
      //     'correctAnswer': correctAnswersCount,
      //     'nextExercise': nextExercise
      //   })
      //
      //   $state.go('result') // Go to result page
      // }, $scope.errorCallback)

      let nextExercise = undefined
      exerciseService.setVal({
        'score': score,
        'totalQuestions': $scope.exercise.interaction.questions.length,
        'correctAnswer': correctAnswersCount,
        'nextExercise': nextExercise
      })

      $state.go('result') // Go to result page

    }

    // Get exercise data and setup
    $scope.$watch('[$ctrl.questionsId,$ctrl.sid]', (n, o) => {
      if ($stateParams.id) {
        $http({ method: 'GET', url: appConstants.api + '/exercise/' + $scope.$ctrl.questionsId + '/' + $scope.$ctrl.sid })
          .then((response) => {
            setupExercise(response.data)
            $localStorage.$reset()
            $localStorage.exercises = []
            $localStorage.questionCount = $scope.exercise.interaction.questions.length
            $localStorage.questions = $scope.exercise.interaction.questions
            $localStorage.currentActiveExercise = 1
            $localStorage.currentActiveQuestion = $scope.step + 1

            // Assign new data
            $localStorage.playlist = [response.data]
            $localStorage.exerciseCount = $localStorage.playlist.length
            $localStorage.currentExercise = response.data.exercise_id
            $localStorage.currentActiveExercise = 1
            $localStorage.questionLink = { type: 'cloze', id: $stateParams.id, sid: $stateParams.sid }

            // $timeout(() => { $state.go('exercise') })
          }, errorCallback)
      } else { // If we have loaded playlist then load the exercise from playlist
        setupExercise($localStorage.playlist.find((item) => item.exercise_id === $localStorage.currentExercise))
      }
    })
  }

}
export default mixComponent
